// This file is made available under Elastic License 2.0.
// This file is based on code available under the Apache license here:
//   https://github.com/apache/incubator-doris/blob/master/fe/fe-core/src/main/java/org/apache/doris/common/publish/ClusterStateUpdate.java

// Licensed to the Apache Software Foundation (ASF) under one
// or more contributor license agreements.  See the NOTICE file
// distributed with this work for additional information
// regarding copyright ownership.  The ASF licenses this file
// to you under the Apache License, Version 2.0 (the
// "License"); you may not use this file except in compliance
// with the License.  You may obtain a copy of the License at
//
//   http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing,
// software distributed under the License is distributed on an
// "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
// KIND, either express or implied.  See the License for the
// specific language governing permissions and limitations
// under the License.

package com.starrocks.common.publish;

import com.google.common.collect.Lists;
import com.starrocks.thrift.TAgentPublishRequest;
import com.starrocks.thrift.TAgentServiceVersion;

import java.util.List;

// State used to publish
public class ClusterStateUpdate {
    private final List<TopicUpdate> updates;

    // Called by builder.
    private ClusterStateUpdate(List<TopicUpdate> updates) {
        this.updates = updates;
    }

    public TAgentPublishRequest toThrift() {
        TAgentPublishRequest request = new TAgentPublishRequest();
        request.setProtocol_version(TAgentServiceVersion.V1);
        for (TopicUpdate update : updates) {
            request.addToUpdates(update.toThrift());
        }
        return request;
    }

    public static Builder builder() {
        return new Builder();
    }

    public static class Builder {
        private List<TopicUpdate> updates;

        public Builder() {
            updates = Lists.newArrayList();
        }

        public Builder addUpdate(TopicUpdate update) {
            updates.add(update);
            return this;
        }

        public ClusterStateUpdate build() {
            return new ClusterStateUpdate(updates);
        }
    }
}
